@@ -1,5 +1,5 @@
 /* xdaliclock - a melting digital clock
- * Copyright (c) 1991-2012 Jamie Zawinski <jwz@jwz.org>
+ * Copyright (c) 1991-2013 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -31,12 +31,7 @@ static void rgb_to_hsv (CGFloat r, CGFloat g, CGFloat b,
 
 
 @interface DaliClockView (ForwardDeclarations)
-
-+ (NSUserDefaultsController *)userDefaultsController;
-+ (void)setUserDefaultsController:(NSUserDefaultsController *)ctl;
-
 - (void)setForeground:(NSColor *)fg background:(NSColor *)bg;
-
 - (void)clockTick;
 - (void)colorTick;
 - (void)dateTick;
@@ -52,23 +47,19 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
   return staticUserDefaultsController;
 }
 
-
-+ (void)initializeDefaults:(NSUserDefaultsController *)controller
++ (void)setUserDefaultsController:(NSUserDefaultsController *)ctl
 {
-  static BOOL initialized_p = NO;
-  if (initialized_p)
-    return;
-  initialized_p = YES;
+  if (staticUserDefaultsController &&
+      staticUserDefaultsController != ctl)
+    [staticUserDefaultsController release];
+  staticUserDefaultsController = [ctl retain];
+}
 
-  staticUserDefaultsController = controller;
-  [controller retain];
 
++ (void)registerDefaults:(NSUserDefaults *)defs
+{
   /* Set the defaults for all preferences handled by DaliClockView.
      (AppController or DaliClockSaverView handles other preferences).
-
-     Depending on class-initialization order, another class (e.g.
-     AppController) might have already put defaults in the preferences
-     object, so make sure we add ours instead of overwriting it all.
    */
   NSColor *deffg = [NSColor  blueColor];
   NSColor *defbg = [NSColor colorWithCalibratedHue:0.50
@@ -115,7 +106,7 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
   NSString *hs = ([test_str hasPrefix:@"13"] ? @"1" : @"0"); // 24- or 12 hour
   [df release];
 
-  NSDictionary* defaults = [NSDictionary dictionaryWithObjectsAndKeys:
+  NSDictionary* extras = [NSDictionary dictionaryWithObjectsAndKeys:
     @"0", @"timeStyle",
     ds,   @"dateStyle",
     hs,   @"hourStyle",
@@ -127,16 +118,9 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
     nil];
 
   NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithCapacity:100];
-
-# ifndef USE_IPHONE
-  [dict addEntriesFromDictionary: [controller initialValues]];
-  [dict addEntriesFromDictionary:defaults];
-  [controller setInitialValues:dict];
-  [[controller defaults] registerDefaults:dict];
-# else  /* USE_IPHONE */
-  [dict addEntriesFromDictionary:defaults];
-  [[NSUserDefaults standardUserDefaults] registerDefaults:dict];
-# endif /* USE_IPHONE */
+  [dict addEntriesFromDictionary:[defs dictionaryRepresentation]];
+  [dict addEntriesFromDictionary:extras];
+  [defs registerDefaults:dict];
 }
 
 
@@ -252,44 +236,51 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
   // Initialize the OpenGL context.
   //
 # ifndef USE_IPHONE
-  NSOpenGLPixelFormatAttribute attrs[] = {
-//    NSOpenGLPFADoubleBuffer,  // does double buffering help at all?
-    NSOpenGLPFAColorSize, 24,
-    NSOpenGLPFAAlphaSize, 8,
-    NSOpenGLPFADepthSize, 0, //16,  we don't need a depth buffer at all.
-    0 };
-  NSOpenGLPixelFormat *pixfmt = [[NSOpenGLPixelFormat alloc] 
-                                  initWithAttributes:attrs];
-  ogl_ctx = [[NSOpenGLContext alloc] 
-              initWithFormat:pixfmt
-                shareContext:nil];
-  check_gl_error ("init ctx");
 
-#  if MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4
-  long svarg;
-#  else  /* 10.5+ */
-  GLint svarg;
-#  endif /* 10.5+ */
+  NSOpenGLContext *ctx = ogl_ctx;
+  if (! ctx) {
+
+    NSOpenGLPixelFormatAttribute attrs[] = {
+      NSOpenGLPFAColorSize, 24,
+      NSOpenGLPFAAlphaSize, 8,
+      NSOpenGLPFADepthSize, 0, //16,  we don't need a depth buffer at all.
+//    NSOpenGLPFADoubleBuffer,  // does double buffering help at all?
+      0 };
+
+    NSOpenGLPixelFormat *pixfmt = 
+      [[NSOpenGLPixelFormat alloc] initWithAttributes:attrs];
+
+    NSAssert (pixfmt, @"unable to create NSOpenGLPixelFormat");
+
+    ctx = [[NSOpenGLContext alloc] 
+            initWithFormat:pixfmt
+              shareContext:nil];
+    ogl_ctx = ctx;
+//    [pixfmt release]; // #### ???
+  }
 
   // Sync refreshes to the vertical blanking interval
-  svarg = 1;
-  [ogl_ctx setValues:&svarg forParameter:NSOpenGLCPSwapInterval];
-  check_gl_error ("init swapinterval");
+  GLint r = 1;
+  [ctx setValues:&r forParameter:NSOpenGLCPSwapInterval];
+//  check_gl_error ("NSOpenGLCPSwapInterval");  // SEGV sometimes. Too early?
 
   // Make window transparency possible
-  svarg = NO;
-  [ogl_ctx setValues:&svarg forParameter:NSOpenGLCPSurfaceOpacity];
-  check_gl_error ("init opacity");
+  r = NO;
+  [ogl_ctx setValues:&r forParameter:NSOpenGLCPSurfaceOpacity];
+//  check_gl_error ("init opacity");
 
 
-  [ogl_ctx makeCurrentContext];
-  check_gl_error ("init makeCurrentContext");
+  [ctx makeCurrentContext];
+  check_gl_error ("makeCurrentContext");
 
   // Clear frame buffer ASAP, else there are bits left over from other apps.
   // #### On 10.7, this causes GL_INVALID_FRAMEBUFFER_OPERATION.
+  // In Dali Clock but not XScreenSaver?
+  //  glClearColor (0, 0, 0, 1);
   //  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   //  check_gl_error ("init clear");
 
+
   // Enable multi-threading, if possible.  This runs most OpenGL commands
   // and GPU management on a second CPU.
   {
@@ -301,7 +292,6 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
     if (err != kCGLNoError) {
       NSLog (@"enabling multi-threaded OpenGL failed: %d", err);
     }
-    check_gl_error ("init threading");
   }
 
   {
@@ -324,45 +314,92 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
 
   check_gl_error ("init");
 
-#else  /* USE_IPHONE */
+# else  // USE_IPHONE
 
-  // Get the layer
-  CAEAGLLayer *eagl_layer = (CAEAGLLayer *)self.layer;
-  eagl_layer.opaque = TRUE;
-  eagl_layer.drawableProperties = 
-    [NSDictionary dictionaryWithObjectsAndKeys:
-      [NSNumber numberWithBool:FALSE], kEAGLDrawablePropertyRetainedBacking,
-      kEAGLColorFormatRGBA8,           kEAGLDrawablePropertyColorFormat,
-      nil];
-  eagl_ctx = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES1];
+  if (!ogl_ctx) {
 
-  if (!eagl_ctx || ![EAGLContext setCurrentContext:eagl_ctx]) {
-    [self release];
-    return nil;
+    CAEAGLLayer *eagl_layer = (CAEAGLLayer *) self.layer;
+    eagl_layer.opaque = TRUE;
+    eagl_layer.drawableProperties = 
+      [NSDictionary dictionaryWithObjectsAndKeys:
+       kEAGLColorFormatRGBA8,           kEAGLDrawablePropertyColorFormat,
+       [NSNumber numberWithBool:FALSE], kEAGLDrawablePropertyRetainedBacking,
+       nil];
+
+    // Without this, the GL frame buffer is half the screen resolution!
+    eagl_layer.contentsScale = [UIScreen mainScreen].scale;
+
+    ogl_ctx = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES1];
   }
 
-  // Without this, the GL frame buffer is half the screen resolution!
-  eagl_layer.contentsScale = [UIScreen mainScreen].scale;
+  if (!ogl_ctx) {
+    [self release];
+    return 0;
+  }
+
+  [EAGLContext setCurrentContext: ogl_ctx];
+
+  if (gl_framebuffer)  glDeleteFramebuffersOES  (1, &gl_framebuffer);
+  if (gl_renderbuffer) glDeleteRenderbuffersOES (1, &gl_renderbuffer);
+
+  glGenFramebuffersOES  (1, &gl_framebuffer);
+  glBindFramebufferOES  (GL_FRAMEBUFFER_OES,  gl_framebuffer);
 
-  glGenFramebuffersOES (1, &gl_framebuffer);
   glGenRenderbuffersOES (1, &gl_renderbuffer);
-  glBindFramebufferOES (GL_FRAMEBUFFER_OES, gl_framebuffer);
   glBindRenderbufferOES (GL_RENDERBUFFER_OES, gl_renderbuffer);
 
-  [eagl_ctx renderbufferStorage:GL_RENDERBUFFER_OES
-            fromDrawable:(CAEAGLLayer*)self.layer];
+// redundant?
+//   glRenderbufferStorageOES (GL_RENDERBUFFER_OES, GL_RGBA8_OES, w, h);
+  [ogl_ctx renderbufferStorage:GL_RENDERBUFFER_OES
+           fromDrawable:(CAEAGLLayer*)self.layer];
 
-  glFramebufferRenderbufferOES (GL_FRAMEBUFFER_OES, GL_COLOR_ATTACHMENT0_OES,
+  glFramebufferRenderbufferOES (GL_FRAMEBUFFER_OES,  GL_COLOR_ATTACHMENT0_OES,
                                 GL_RENDERBUFFER_OES, gl_renderbuffer);
 
-  if (glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES) !=
-      GL_FRAMEBUFFER_COMPLETE_OES) {
-    NSLog (@"incomplete framebuffer");
-    abort();
+  int err = glCheckFramebufferStatusOES (GL_FRAMEBUFFER_OES);
+  switch (err) {
+  case GL_FRAMEBUFFER_COMPLETE_OES:
+    break;
+  case GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_OES:
+    NSAssert (0, @"framebuffer incomplete attachment");
+    break;
+  case GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_OES:
+    NSAssert (0, @"framebuffer incomplete missing attachment");
+    break;
+  case GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_OES:
+    NSAssert (0, @"framebuffer incomplete dimensions");
+    break;
+  case GL_FRAMEBUFFER_INCOMPLETE_FORMATS_OES:
+    NSAssert (0, @"framebuffer incomplete formats");
+    break;
+  case GL_FRAMEBUFFER_UNSUPPORTED_OES:
+    NSAssert (0, @"framebuffer unsupported");
+    break;
+/*
+  case GL_FRAMEBUFFER_UNDEFINED:
+    NSAssert (0, @"framebuffer undefined");
+    break;
+  case GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER:
+    NSAssert (0, @"framebuffer incomplete draw buffer");
+    break;
+  case GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER:
+    NSAssert (0, @"framebuffer incomplete read buffer");
+    break;
+  case GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE:
+    NSAssert (0, @"framebuffer incomplete multisample");
+    break;
+  case GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS:
+    NSAssert (0, @"framebuffer incomplete layer targets");
+    break;
+ */
+  default:
+    NSAssert (0, @"framebuffer incomplete, unknown error 0x%04X", err);
+    break;
   }
 
   check_gl_error ("OES_init");
-#endif /* USE_IPHONE */
+
+# endif // USE_IPHONE
 
   glGenTextures (2, textures);
   check_gl_error ("glGenTextures");
@@ -868,7 +905,7 @@ FAIL:
     NSRectFill ([self frame]);
   }
 }
-#endif /* USE_IPHONE */
+#endif /* !USE_IPHONE */
 #endif /* 0 */
 
 
@@ -996,7 +1033,7 @@ FAIL:
   // Set the viewport and projection matrix
 
 #ifdef USE_IPHONE
-  [EAGLContext setCurrentContext:eagl_ctx];
+  [EAGLContext setCurrentContext:ogl_ctx];
   glBindFramebufferOES(GL_FRAMEBUFFER_OES, gl_framebuffer);
 #else /* !USE_IPHONE */
   [ogl_ctx makeCurrentContext];
@@ -1115,7 +1152,7 @@ FAIL:
   check_gl_error ("finish");
 #else /* USE_IPHONE */
   glBindRenderbufferOES (GL_RENDERBUFFER_OES, gl_renderbuffer);
-  [eagl_ctx presentRenderbuffer:GL_RENDERBUFFER_OES];
+  [ogl_ctx presentRenderbuffer:GL_RENDERBUFFER_OES];
   check_gl_error ("finish");
 #endif /* USE_IPHONE */
 }
@@ -1194,11 +1231,13 @@ FAIL:
   // See comment in viewDidMoveToWindow -- doing this there caused errors.
   // 
   if ([self window] && ownWindow) {
+    ownWindow = 0; // only do it once
     [[self window] setBackgroundColor:[NSColor clearColor]];
     [[self window] setOpaque:NO];
-    [[NSColor clearColor] set];
-    NSRectFill ([self frame]);
-    ownWindow = 0; // only do it once
+    // Not sure why I was doing this, but this causes the System Preferences
+    // window to have random black areas drawn on it.
+    //  [[NSColor clearColor] set];
+    //  NSRectFill ([self frame]);
   }
 # endif /* !USE_IPHONE */
 
@@ -1251,7 +1290,7 @@ FAIL:
                 forKey:@"initialForegroundColor"];
     [controller setObject:[NSArchiver archivedDataWithRootObject:bg]
                 forKey:@"initialBackgroundColor"];
-# endif /* !USE_IPHONE */
+# endif /* USE_IPHONE */
   }
 
   /* re-schedule the timer according to current fps.
@@ -1528,14 +1567,12 @@ FAIL:
 }
 
 
-- (void)setInitialForegroundColor:(NSColor *)anInitialForegroundColor
+- (void)setInitialForegroundColor:(NSColor *)c
 {
-  if (initialForegroundColor != anInitialForegroundColor) {
+  if (initialForegroundColor != c) {
     if (initialForegroundColor)
       [initialForegroundColor release];
-    initialForegroundColor = (anInitialForegroundColor
-                              ? [anInitialForegroundColor retain]
-                              : 0);
+    initialForegroundColor = (c ? [c retain] : 0);
     [self setForeground:initialForegroundColor
              background:initialBackgroundColor];
   }
@@ -1548,14 +1585,12 @@ FAIL:
 }
 
 
-- (void)setInitialBackgroundColor:(NSColor *)anInitialBackgroundColor
+- (void)setInitialBackgroundColor:(NSColor *)c
 {
-  if (initialBackgroundColor != anInitialBackgroundColor) {
+  if (initialBackgroundColor != c) {
     if (initialBackgroundColor)
       [initialBackgroundColor release];
-    initialBackgroundColor = (anInitialBackgroundColor
-                              ? [anInitialBackgroundColor retain]
-                              : 0);
+    initialBackgroundColor = (c ? [c retain] : 0);
     [self setForeground:initialForegroundColor
              background:initialBackgroundColor];
   }
